@@ -3,10 +3,9 @@ import time
 import markdown as md
 import sys
 import traceback
+import glob
 
 from docutils.core import publish_parts
-from Page import Page
-from DirectoryWalker import DirectoryWalker
 from jinja2 import Environment, FileSystemLoader, meta, BaseLoader, TemplateNotFound, ChoiceLoader, Template, Markup
 from SilentUndefined import SilentUndefined
 from MarkdownExtension import MarkdownExtension
@@ -33,10 +32,9 @@ def filter_template_dir(filename):
         return False;
     return True;
 
+
 class JinjaStaticRenderer():
-    """
-    In theory this should probably sub-class a generic renderer
-    """ 
+
     def __init__(self, config):
         self.config = config
         # Setup the env
@@ -52,15 +50,11 @@ class JinjaStaticRenderer():
         
         self.env.filters["markdown"] = markdown_filter
         self.env.filters['rst'] = rst_filter        
-        self.template_modified_times = {}
         self.template_trees = {}
-
-        # self.template_modified_times = self.get_template_mtimes(config)
-        # self.template_trees = self.load_abstract_syntax_trees()
-
+        self.template_modified_times = {}
+        
         var_path=os.path.dirname(self.config['compiled_templates_file'])
 
-
         if self.config["verbose"] == True:
             print "Ensure directory exists: \033[34m%s\033[0m" % (var_path)
             print "Ensure directory exists: \033[34m%s\033[0m" % (config['output_dir'])
@@ -78,35 +72,6 @@ class JinjaStaticRenderer():
             print "Loading templates from: \033[34m%s\033[0m" % (config['templates_dir'])
             print "Compile templates to .zip: \033[32m%s\033[0m" % (self.config['compiled_templates_file'])
 
-        # This lets us have bespoke template pages in the input dir too. 
-        # self.env.loader = FileSystemLoader(self.config['templates_dir'])
-
-    # def get_template_mtimes(self, config):
-    #     """
-    #     Get the template modified times to save disk lookups for each item.
-    #     """
-    #     template_files = self.env.list_templates()
-
-    #     mod_times = {}
-
-    #     for path in template_files:
-    #         file_name = path
-    #         file_mtime = os.path.getmtime("%s/%s" % (self.config['templates_dir'], file_name)  )
-    #         mod_times[file_name] = file_mtime
-
-    #     return mod_times
-
-    # def load_abstract_syntax_trees(self):
-    #     """
-    #     Cache up the ASTs so we don't have to hit the disk for each page.
-    #     So far, this is probably the single biggest speed boost.
-    #     """
-    #     template_files = self.env.list_templates()
-    #     asts = {}
-    #     for template in template_files:
-    #         asts[template] = self.env.parse(self.env.loader.get_source(self.env, template))
-
-    #     return asts
 
     def get_template_mtime(self, template):
         """
@@ -136,112 +101,119 @@ class JinjaStaticRenderer():
 
         return self.template_trees[template]
 
-    def get_global_vars(self):
-        return self.config['site_vars']
-
     def ensure_dir(self, dir):
         try:
             os.stat(dir)
         except:
             os.makedirs(dir)
 
+
     def render(self, data_source):
-        """
-        Kick off the rendering loop. If no cache found, or config has been modified, 
-        force everything to be re-rendered.
-        """
-        force_render = 0
-        last_build_time = self.get_last_build_time()
-        config_modified_time = self.get_config_modified_time()
-        output_dir = self.config['output_dir']
-        output_file_extension = self.config['output_file_extension']
-        templates_dir = self.config['templates_dir']
-        default_template = self.config['default_template']
-        
-        if last_build_time == 0 or config_modified_time >= last_build_time:
-            force_render = 1
+        self.log_data = []
 
-        if self.config["force"] == True:
-            force_render = 1
-
-        rendered_files = []
-        
         for item in data_source:
-            item.set_output_path(output_dir, output_file_extension)
-            rendered_file = self.render_item(last_build_time, self.env, force_render, templates_dir, default_template, item)
-            rendered_files.append(rendered_file)
+            template = self.load_template(item.get_template())
 
+            if (self.should_render_item(item, template)):
+                item.before_render()
 
+                if self.config["verbose"] == True:
+                    print "\033[34m%s\033[37m last built at: %s\033[0m" % (item.get_file_path(), item.get_modified_time())
+                self.save(self.render_template(template, item));
 
         self.set_last_build_time(time.time())
-        self.update_log(rendered_files, last_build_time)
+        self.update_log(self.log_data, self.get_last_build_time())
 
 
-    def render_item(self, last_build_time, env, force_render, templates_dir, default_template, page):
-        new_file_path = page.get_output_path()
-        new_directory = os.path.split(new_file_path)[0]
-        
-        # Force a render if the output file doesn't exist
-        if not os.path.exists(new_file_path) or page.get_modified_time() >= last_build_time:
-            force_render = 1
+    def render_template(self, template, item):
+        return (template.render(data=item.get_page_content(), meta=item.get_metadata()), item)
 
-        template_modified_time = 0
-        template = page.get_template()
-        
-        if template == 0:
-            template = default_template
 
+    def save(self, data):
+        item=data[1]
+        html=data[0]
+
+        filename = item.get_output_path()
+        file_object = open(filename, "w")
+        new_directory = os.path.split(filename)[0]
+
+        try: 
+            self.ensure_dir(new_directory)
+            file_object.write(html.encode('utf8'))
+            self.print_stdout(item.get_file_path(), filename, item.get_template())
+            item.on_save()
+        except:
+            print "\033[31mCouldn't write %s\033[0m" % (filename)
+            traceback.print_exc()
+            self.log_item_saved(item.get_file_path(), item.get_template(), 0)
+            return False
+        finally:
+            self.log_item_saved(item.get_file_path(), item.get_template(), 1)
+            return True
+
+
+    def load_template(self, template_name):
+        template_object = False
         try:
-            template_object = env.get_template(template)
+            template_object = self.env.get_template(template_name)
         except:
             print "\033[1;91mCouldn't parse `%s`. Check the template for errors \033[0;37m" % (template)
             
             if self.config["verbose"] == True:
                 traceback.print_exc()
             print "\033[0m"
-            return [new_file_path, 0, template]
-
-        referenced_templates = self.get_referenced_templates(template)
-        template_modified_time = self.get_template_mtime(template)
-
-        if template_modified_time >= last_build_time:
-            force_render = 1
-
-        ref_template_mtime = 0
-        for t in referenced_templates:
-            time = self.get_template_mtime(t)
-            if time >= last_build_time:
-                force_render = 1
-                ref_template_mtime = time
-
-        if self.config["verbose"] == True:
-            print "\033[34m%s\033[37m last built at: %s, page mtime: %s, templ mtime: %s, ref-templates mtime: %s\033[0m" % (page.file_path, last_build_time, page.get_modified_time(), template_modified_time, ref_template_mtime)
+            return False
+        finally:
+            return template_object
 
 
-        file_did_render = 0
+    def should_render_item(self, item, template):
+        force_render = 0
+        last_build_time = self.get_last_build_time()
+        output_path = item.get_output_path();
 
-        if template_object and last_build_time == 0 or force_render == 1:
-            self.ensure_dir(new_directory)
-            html = template_object.render(
-                data=page.get_page_content(),
-                site=self.get_global_vars(),
-                meta=page.get_page_vars(),
-                )
-            new_file = open(new_file_path, "w")
-            try: 
-                new_file.write(html.encode('utf8'))
-            except:
-                print "\033[31mTrouble writing %s\033[0m" % (new_file_path)
-                traceback.print_exc()
+        if last_build_time == 0:
+            return True
+
+        config_modified_time = self.get_config_modified_time()
+        
+        if last_build_time == 0 or config_modified_time >= last_build_time:
+            return True
+
+        if self.config["force"] == True:
+            return True
+
+        if output_path and not os.path.exists(output_path):
+            return True
+        
+        if item.get_modified_time() >= last_build_time:
+            return True
+
+        template_modified_time = 0
+
+
+        if template:
+
+            referenced_templates = self.get_referenced_templates(template)
+            template_modified_time = self.get_template_mtime(item.get_template())
+
+            if template_modified_time >= last_build_time:
+                return True
+
+            ref_template_mtime = 0
+            for t in referenced_templates:
+                time = self.get_template_mtime(t)
+                if time >= last_build_time:
+                    ref_template_mtime = time
+                    return True
+
+        return False
 
-            self.print_stdout(page.get_file_path(), new_file_path, template)
-            file_did_render = 1
 
-        return [new_file_path, file_did_render, template]
-            
     def print_stdout(self, original_path, new_path, template):
         print "\033[1;32mBuilding \033[0m\033[2m%s\033[0m > \033[34m%s \033[2m[%s]\033[0m" % (original_path, new_path, template)
 
+
     def update_log(self, items, last_modified_time):
         file = open(self.config['compiled_templates_log'], 'w')
         print>>file, "this_render=%s, prev_render=%s" % (time.time(), last_modified_time)
@@ -251,6 +223,11 @@ class JinjaStaticRenderer():
 
         if self.config["verbose"] == True:
             print "Wrote log to: \033[32m%s\033[0m" % (self.config['compiled_templates_log'])
+    
+    def log_item_saved(self, path, template, result):
+        if not self.log_data:
+            self.log_data = []
+        self.log_data.append([path, template, result])
 
     def get_last_build_time(self):
         """
@@ -281,27 +258,11 @@ class JinjaStaticRenderer():
         with open(self.config['build_cache_file'], "w") as buildFile:
             buildFile.write("%s" % (update_time))
 
-    def get_data(self, source_type, params):
-        """
-        Future-friendly method. For when this interfaces with a swanky API.
-        """
-        if source_type == "directory":
-            source = DirectoryWalker(params)
-            return source.fetch()
+    def get_global_vars(self):
+        return self.config["site_vars"]
 
-    def go(self):
+    def go(self, data):
         """
         Request some data from an endpoint/directory
         """
-        data = self.get_data(
-            "directory",
-            {
-                'input_dir': self.config['input_dir'],
-                'ignore_files': self.config['ignore'],
-                'item_class': Page,
-                'data_format': self.config['data_format'],
-                'nocache': self.config['nocache']
-            }
-        )
-
         self.render(data)
\ No newline at end of file
